/***************************************************************************
 *   Copyright (C) 2006-2015 Helge Kortendieck <alan@kasmithee.de>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef _MAGICK_H_
#define _MAGICK_H_

#include <QByteArray>
#include <QFile>
#include <QFileInfo>
#include <QObject>
#include <QProcess>
#include <QString>
#include <QStringList>
#include <QTimer>

#include <iostream>		// debug
using namespace std;	// debug

class magick : public QObject
{
	Q_OBJECT

public:
	magick( QObject *parent = 0 );
		~magick();

private:
	bool bD;
	bool bD2;
	bool bApH;					// flag append horizontally (true) or vertically (false)
	bool bErr;					// flag whether we have error while running magick
	bool bPJ;					// flag, create Pdf or Jpg

//	int i;						// iteration counter
	int iAppendCnt;				// number of pics to append
	int iAppendIter;			// append iterations
//  int iArgs;					// counter for arguments, we need at least 3!
	int iD;
	int iChk;					// check for convert: 0=installed, 1=could not be found
	int iMod;					// mod of 2 from images count - even or odd, if odd, we need to just copy the last image without appending
	int iProc;					// process type: 0=none, 1=append, 2=convert

	QByteArray aOutput;			// stdout from qprocess

	QProcess *mag;				// magick qprocess

	QString sAppend1;
	QString sAppend2;
	QString sCmd;				// command for qprocess
	QString sCompress;			// compress type
	QString sD;
	QString sD2;
	QString sD3;
	QString sD4;
	QString sErrT;				// error message title
	QString sErr;				// error message
	QString sIn;				// input line from qprocess output
	QString sLine;				// command line
	QString sMTmpDir;			// full path & name of temp dir
	QString sPad;				// 0 padding for picture numbering
	QString sMOutputFile;		// full path n name of output file
	QString sQuality;			// image quality
	QString sSuffix;			// suffix of 1st file to append

	QStringList slD;
	QStringList slArgs;			// qprocess argument(s)
	QStringList slToProcess;   	// list with images to process, append left to right or convert into 1 pdf
	QStringList slResult;		// list with images after appending 1 to 1, left to right

private slots:
	void magickChk();																		// check for ImageMagick convert command
	void magickAppendPrep(const QString& sD, const QStringList& slD, bool bD, bool bD2);	// prepare qprocess: convert append, concatenate every 2 images into 1
	QString magickAppendCreatePicName();													// create name of concatenated pic
	void magickToPdfPrep(const QString& sD, const QStringList& slD, const QString& sD2, const QString& sD3, const QString& sD4);	// prepare converting images into one pdf

	void procPrep();			// prepare convert run
	void procStart();			// start convert qprocess
	void procErr();				// error, during convert run, send error message
	void procExit(int iD, QProcess::ExitStatus iD2);	// rsync process exited
	void procStop();			// stop convert process
	void procReadStdErr();		// read error output from convert process
	void procReadStdOut();		// read output from convert process

	signals:
		void sigChkEnd(int iD);								// checking for ImageMagick convert. Result: 0=not found, 1=found
		void sigMagickAppendEnded(const QStringList slD);	// append ended, send list with resulting images or empty list on error
		void sigMagickToPdfEnded(const QString& sD);		// converting picture(s) to one pdf file has ended; sD=tmpdir, sD=""=error
/*    void sigErr( const QString& sErrT, const QString& sErr );     // rsync failed
    void sigOk( const QString& sD, int iD, const QString& sErr ); // rsync appeared to be ended successfully: last run date, error, warning string
    void sigStopped();                                            // rsync has been stopped by user request, rsync process has ended
*/
};

#endif // _MAGICK_H_
