/****************************************************************************
 **
 **  Copyright (C) 2007-2015 Helge Kortendieck, alan@kasmithee.de
 **
 **  Main program, set number of words to test in test mode and
 **  handling vocabulary files: creating, changing...
 **
 **  This file may be used under the terms of the GNU General Public
 **  License version 2.0 or later as published by the Free Software Foundation
 **
 **  This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 **  WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ****************************************************************************/

#include "qpictopdf.h"

Qptp::Qptp(QMainWindow *parent)
		: QMainWindow(parent)
{
	bQuit = false;
//	iCreate = 0;
	iRunState = 0;
	iWShown = 0;
	iWToShow = 0;
	sTmpDir = "";
	sFileOld = "-1";

	actionsCreate();

	ui.setupUi(this);
		QButtonGroup *bgOutputType = new QButtonGroup();
			bgOutputType->addButton(ui.rB_Pdf);
			bgOutputType->addButton(ui.rB_Jpg);

	menuBarCreate();
	toolBar = addToolBar(tr("Show toolbar"));
	toolBarCreate();
	statusBarCreate();

	settingsRead();

	connect(ui.tV_FileSys, SIGNAL(expanded(const QModelIndex &)), this, SLOT(dmColResize(const QModelIndex &)));
	connect(ui.tV_FileSys, SIGNAL(pressed(const QModelIndex &)), this, SLOT(dmSelectionChanged(const QModelIndex &)));
	connect(ui.rB_Pdf, SIGNAL(toggled(bool)), this, SLOT(outTypeToggled()));
	connect(ui.lE_Path, SIGNAL(editingFinished()), this, SLOT(destDirEditFin()));
	connect(ui.pB_Path, SIGNAL(clicked()), this, SLOT(destDirSetClicked()));
	connect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
	connect(ui.pB_File, SIGNAL(clicked()), this, SLOT(outNameDialogClicked()));
	ui.pB_Path->setIcon(QIcon("/usr/share/qpictopdf/icons/scalable/open.svg"));
	ui.pB_File->setIcon(QIcon("/usr/share/qpictopdf/icons/scalable/open.svg"));

	initDelay();								// short timer to allow GUI to come up before proceeding...
}

Qptp::~Qptp()
{
}

//  **********  settings  **********
void Qptp::settingsRead()
{
	QSettings settings("qpictopdf", "qpictopdf");

	settings.beginGroup("mainwindow");
		QPoint pos = settings.value("pos", QPoint(0, 0)).toPoint();
		QSize size = settings.value("size", QSize(740, 540)).toSize();
	settings.endGroup();

	settings.beginGroup("outputtype");
		ui.rB_Pdf->setChecked(settings.value("outputpdf", "true").toBool());
		if(ui.rB_Pdf->isChecked() == false) {
			ui.rB_Jpg->setChecked(true);
		}
		ui.cB_PdfConcPic->setChecked(settings.value("pdfconcpic", "false").toBool());
		ui.coB_PdfCompType->setCurrentIndex(settings.value("pdfcomptype", "1").toInt());
		ui.spB_PdfCompQual->setValue(settings.value("pdfquality", "30").toInt());
		ui.rB_JpgH->setChecked(settings.value("jpgconch", "true").toBool());
		if(ui.rB_JpgH->isChecked() == false) {
			ui.rB_JpgV->setChecked(true);
		}
	settings.endGroup();

	settings.beginGroup("outputpathname");
		sOutDir = settings.value("path", "/").toString();
		sLastDir = settings.value("lastdir", "/").toString();
		ui.cB_InSrcDir->setChecked(settings.value("insrcdir", "false").toBool());
		ui.cB_SugName->setChecked(settings.value("sugname", "true").toBool());
		ui.cB_DelOrigPics->setChecked(settings.value("delorigpics", "false").toBool());
	settings.endGroup();

	resize(size);
	move(pos);
}

void Qptp::settingsWrite()
{
	QSettings settings("qpictopdf", "qpictopdf");

	settings.beginGroup("mainwindow");
		settings.setValue("pos", pos());
		settings.setValue("size", size());
	settings.endGroup();

	settings.beginGroup("outputtype");
		settings.setValue("outputpdf",ui.rB_Pdf->isChecked());
		settings.setValue("pdfconcpic", ui.cB_PdfConcPic->isChecked());
		settings.setValue("pdfcomptype", ui.coB_PdfCompType->currentIndex());
		settings.setValue("pdfquality", ui.spB_PdfCompQual->value());
		settings.setValue("jpgconch",ui.rB_JpgH->isChecked());
	settings.endGroup();

	settings.beginGroup("outputpathname");
		settings.setValue("path", ui.lE_Path->text());
		settings.setValue("lastdir", sLastDir);
		settings.setValue("insrcdir", ui.cB_InSrcDir->isChecked());
		settings.setValue("sugname", ui.cB_SugName->isChecked());
		settings.setValue("delorigpics", ui.cB_DelOrigPics->isChecked());
	settings.endGroup();

	settings.sync();
}

//  **********  actions, menu, toolbar, statusbar  **********
void Qptp::actionsCreate()
{
	aboutAct = new QAction(QIcon("/usr/share/qpictopdf/icons/scalable/about.svg"), tr("&About qpictopdf"), this);
	aboutAct->setShortcut(tr("Ctrl+A"));
	aboutAct->setToolTip(tr("Show the application's about information"));
	connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));
	
	createAct = new QAction(QIcon("/usr/share/qpictopdf/icons/scalable/create.svg"), tr("&Create pdf"), this);
	createAct->setShortcut (tr("Ctrl+C"));
	createAct->setToolTip(tr("Create output from selected picture(s)"));
	connect(createAct, SIGNAL(triggered()), this, SLOT(clickedCreate()));

	disabledAct = new QAction(QIcon("/usr/share/qpictopdf/icons/scalable/disabled.svg"), tr("Creating pdf not possible"), this);
	disabledAct->setToolTip(tr("Creating output not possible - see bottom of screen"));
	disabledAct->setStatusTip(tr("Creating output not possible - convert (from ImageMagick) not found, output directory invalid, nothing selected or no output file name specified."));

	exitAct = new QAction(QIcon("/usr/share/qpictopdf/icons/scalable/exit.svg"), tr("&Quit the program"), this);
	exitAct->setShortcut(tr("Ctrl+Q"));
	exitAct->setToolTip(tr("Quit the application"));
	connect(exitAct, SIGNAL(triggered()), this, SLOT(quitPrep()));

	helpAct = new QAction(QIcon("/usr/share/qpictopdf/icons/scalable/help.svg"), tr("&Help"), this);
	helpAct->setShortcut(tr("Ctrl+H"));
	helpAct->setToolTip(tr("Show the application's online help"));
	connect(helpAct, SIGNAL(triggered()), this, SLOT(help()));

	refreshAct = new QAction(QIcon("/usr/share/qpictopdf/icons/scalable/refresh.svg"), tr("&Refresh dir view"), this);
	refreshAct->setShortcut(tr("Ctrl+R"));
	refreshAct->setToolTip(tr("Refresh file browser view"));
	connect(refreshAct, SIGNAL(triggered()), this, SLOT(dmRefresh()));

	stopAct = new QAction(QIcon("/usr/share/qpictopdf/icons/scalable/stop.svg"), tr("St&op pdf creation"), this);
	stopAct->setShortcut(tr("Ctrl+S"));
	stopAct->setToolTip(tr("Stop creating the pdf - currently not implemented !"));
//	connect(stopAct, SIGNAL(triggered()), this, SLOT(clickedStop()));
}

void Qptp::menuBarCreate()
{
	menuBar()->clear();
	fileMenu = menuBar()->addMenu(tr("&File"));
		fileMenu->addAction(refreshAct);
		fileMenu->addSeparator();
		fileMenu->addAction(exitAct);
	menuBar()->addSeparator();
	pdfMenu = menuBar()->addMenu(tr("&Output"));
		if(iRunState == 0) {					// convert not found or output dir invalid
			pdfMenu->addAction(disabledAct);
		} else if(iRunState == 1) {				// output creation enabled
			pdfMenu->addAction(createAct);
		} else if(iRunState == 2) {				// output creation running
			pdfMenu->addAction(stopAct);
		}
	menuBar()->addSeparator();
	helpMenu = menuBar()->addMenu(tr("&Help"));
		helpMenu->addAction(helpAct);
		helpMenu->addAction(aboutAct);
}

void Qptp::toolBarCreate()
{
	toolBar->clear();
	toolBar->addAction(exitAct);
	toolBar->addSeparator();
	if(iRunState == 0) {					// convert not found or output dir invalid
		toolBar->addAction(disabledAct);
	} else if(iRunState == 1) {				// output creation enabled
		toolBar->addAction(createAct);
	} else if(iRunState == 2) {				// output creation running
		toolBar->addAction(stopAct);
	}
	toolBar->addSeparator();
	toolBar->addAction(refreshAct);
}

void Qptp::statusBarCreate()
{
	statusBarUpd(tr("Ready"));
}

void Qptp::statusBarUpd(const QString& sD)
{
	statusBar()->showMessage(sD);
}

void Qptp::updMnTlBr()						// update menu and toolbar
{
	menuBarCreate();
	toolBarCreate();
}

//  **********  message boxes  **********
void Qptp::messageBox(int iD, const QString sD, const QString sD2)	// show QMessageBox at cursor position
{
	getCursorPos();
	QMessageBox msgBox;
	msgBox.setWindowTitle(sD);
	if(iD == 1) {
		msgBox.setIcon(QMessageBox::Information);
	} else if(iD == 2) {
		msgBox.setIcon(QMessageBox::Warning);
	} else if(iD == 3) {
		msgBox.setIcon(QMessageBox::Critical);
	} else {
		return;
	}
	msgBox.setText(sD2);
	msgBox.addButton(tr("&OK"), QMessageBox::AcceptRole);
	msgBox.move(iX, iY);
	msgBox.exec();
}

void Qptp::getCursorPos()					// get cursor position - iX=x , iY=y
{
	qPos = QCursor::pos();
	iX = qPos.x() - 100;
	if(iX < 0) {
		iX = 0;
	}
	iY = qPos.y() - 175;
	if(iY < 0) {
		iY = 0;
	}
}

// ********************************************************************************************************************
// **  about/help/error widget handling  **
// ****************************************
void Qptp::about()							// about clicked
{
	wCtrl(1);
}

void Qptp::help()							// help clicked
{
	wCtrl(2);
}

void Qptp::ahwCreate()						// create about/help/error dialog widget
{
	if(iWShown == 0) {
		ahw = new aboutHelp();
		connect(this, SIGNAL(sigAhwShow(int)), ahw, SLOT(dlgShow(int)));
		connect(ahw, SIGNAL(sigDelAhw()), this, SLOT(ahwDel()));
	}
}

void Qptp::ahwDel()							// ahw widget was been hidden - delete it
{
	ahw->deleteLater();
	iWShown = 0;
	if(bQuit) {
		quitPrep();
	}
}

//  **********  quit  **********
void Qptp::quitPrep()                       // quit clicked clean up
{
	getCursorPos();
	QMessageBox msgBox;
	msgBox.setWindowTitle(tr("Quit ?"));
	msgBox.setIcon(QMessageBox::Question);
	msgBox.setText(tr("\nSure to quit ?\n"));
	msgBox.addButton(tr("Yes, &Quit"), QMessageBox::AcceptRole);
	msgBox.addButton(tr("No, &Cancel"), QMessageBox::RejectRole);
	msgBox.move(iX, iY);
	if(msgBox.exec() == 0) {
		bQuit = true;
		settingsWrite();
		tmpDirDel();
		quitDelay();
	}
}

void Qptp::quitDelay()                      // delay to allow clean up - needed ????????????????????
{
	QTimer::singleShot(500, this, SLOT(quitDo()));
}

void Qptp::quitDo()                         // finally quit the program
{
	exit(0);
}

//  **********  init  **********
void Qptp::initDelay()
{
	QTimer::singleShot(200, this, SLOT(init()));
}

void Qptp::init()
{
	outputSetType();
	magickCreate();
}

void Qptp::initCont()						// continue after checking if ImageMagick's convert is accessable
{
	destDirSet(sOutDir);
	dmCreate();
}

//  **********  DirModel  **********
void Qptp::dmCreate()
{
	statusBarUpd(tr("Just a moment, creating directory tree..."));
	model = new QDirModel();
		model->setSorting(QDir::SortFlags(QDir::DirsFirst || QDir::Name || QDir::IgnoreCase));
		model->sort(0, Qt::SortOrder(Qt::AscendingOrder));
	ui.tV_FileSys->setModel(model);
	ui.tV_FileSys->setRootIndex(model->index("/"));
	ui.tV_FileSys->sortByColumn (0, Qt::SortOrder(Qt::AscendingOrder));
	for(iD = 0; iD < 4; ++iD) {
		ui.tV_FileSys->resizeColumnToContents(iD);
	}
	ui.tV_FileSys->setSelectionBehavior(QAbstractItemView::SelectionBehavior(QAbstractItemView::SelectItems));
	ui.tV_FileSys->setSelectionMode(QAbstractItemView::SelectionMode(QAbstractItemView::ExtendedSelection));
	dmLastDirExpand();

	sD = ui.lE_Path->text();					// **************** needed?
	statusBarUpd(tr("Ready"));
}

void Qptp::dmColResize(const QModelIndex miIndex)	// auto resize columns
{
	for(iD = 0; iD < 4; ++iD) {
		ui.tV_FileSys->resizeColumnToContents(iD);
	}
}

void Qptp::dmLastDirExpand()				// expand to dir used in last session
{
	QDir dir(sLastDir);
	do {
		if(dir.exists() == true) {			// old dir still exists, expand
			miIndex = model->index(dir.absolutePath(), 0);
			ui.tV_FileSys->expand(miIndex);
			dmColResize(miIndex);
		}
	} while(dir.cdUp() == true);
	connect(ui.tV_FileSys, SIGNAL(clicked(const QModelIndex &)), this, SLOT(dmLastDirSet(const QModelIndex &)));
}

void Qptp::dmLastDirSet(const QModelIndex qmiD)	// set last used dir
{
	QFileInfo fi2(model->filePath(qmiD));
	sLastDir = fi2.path();
}

void Qptp::dmRefresh()						// refresh file system gui
{
	model->refresh();
	dmLastDirExpand();
}

void Qptp::dmSelectionChanged(const QModelIndex miD)	// selection in tree view has changed
{
	lDirModSelIndexes = ui.tV_FileSys->selectionModel()->selectedIndexes();	// get selection
	if(lDirModSelIndexes.count() == 0) {									// nothing selected, cannot create output
		runStateReset();
		return;
	}
	
	sSelItem0Path = model->filePath(lDirModSelIndexes[0]);					// get full path and name of first selected item in tree view
	QFileInfo fi1(sSelItem0Path);
	if(fi1.isDir() == true) {												// dir selected - no go, disable
		runStateReset();
		return;
	}

	runStateReset();

	sSelItem0Name = model->fileName(lDirModSelIndexes[0]);					// construct suggestion
	dmLastDirSet(lDirModSelIndexes[0]);

	if(ui.cB_SugName->isChecked() == true) {
		pdfNameSuggest();
	}
}

void Qptp::dmGetSelectedPics()				// get full path and name of pictures selected in the listview
{
	slPicsSel.clear();
	slPicsToPdf.clear();
	for(iD = 0; iD < lDirModSelIndexes.count(); ++iD) {
		slPicsSel.append(model->filePath(lDirModSelIndexes[iD]));
	}
}

// ****************************************************************
// **  set destination dir dialog slots                          **
// ****************************************************************
void Qptp::destDirSetClicked()				// icon to set destination dir clicked
{
  destDirGetStartDir();

  sD = "";
  QFileDialog fd(this);
    fd.setFileMode(QFileDialog::Directory);
    fd.setViewMode(QFileDialog::List);
    fd.setNameFilter(tr("ALL (*.*)"));
    fd.setDirectory(sDir);
    if(fd.exec()) {
		slD = fd.selectedFiles();
		sOutDir = destDirStripDir(slD.at(0));
		destDirSet(sOutDir);
    }
}

void Qptp::destDirEditFin()					// manually setting the pdf path finished
{
	destDirSet(destDirStripDir(ui.lE_Path->text()));
}

bool Qptp::destDirChkWri(const QString& sD)	// check if dir is writable
{
	sOutDir = sD;
	if(sOutDir != "") {
		QFileInfo fi(sOutDir);
		if(fi.isDir() == false) {
			sOutDir = fi.absolutePath();
			fi.setFile(sOutDir);
		}
//cout << "destDirChkWri() : fi.absolutePath " << sOutDir.toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
		if(fi.isWritable() == true) {
			return true;
		}
	}
	iRunState = 1;
	updMnTlBr();
	return false;
}

void Qptp::destDirSet(const QString& sD)	// set dest dir in line widget, home dir if invalid
{
	if(destDirChkWri(sD) == true) {
		sOutDir = ensurePathSlash(sOutDir);
	} else {
		messageBox(2, tr("Error"), tr("The selected directory\n%1\nis not writable!\nOutput dir set to your home directory.").arg(sOutDir));
		sOutDir = ensurePathSlash(QDir::homePath());
	}
	ui.lE_Path->setText(sOutDir);
}

QString Qptp::destDirStripDir(const QString& sD)	// get path from dir or file
{
	QFileInfo fi2(sD);
	if(fi2.isDir() == true) {
		QDir di(sD);
		return di.absolutePath();
	} else {
		return fi2.absolutePath();
	}
}

void Qptp::destDirGetStartDir()				// get start dir for set dir dialog
{
	sDir = ui.lE_Path->text();
	QFileInfo fi(sDir);						// get start dir for dialog
	if(fi.isDir() == false) {
		sDir = fi.absolutePath();
	}
	QFileInfo fi2(sDir);
	if(fi2.exists() == false or fi2.isWritable() == false) {
		sDir = QDir::homePath();
	}
}

// **********************************************************
// **  set destination file name                           **
// **********************************************************
void Qptp::outNameDialogClicked()			// icon to open file dialog clicked
{
	sFile = QFileDialog::getSaveFileName(this, tr("Set pdf name"), ui.lE_Path->text(), tr("*.pdf"));
	if(sFile != "") {							// when user did not press cancel
		QFileInfo fi(sFile);
		sFile = fi.fileName();						// just get the file name!
//cout << "outNameDialogClicked() : sFile = " << sFile.toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
//		outNameSetFormat();
		outNameSet();
	}
}

void Qptp::outNameLeEditFin()				// editing output file name finished
{
	sFile = ui.lE_File->text();
	if(sFile != sFileOld) {
		outNameSetFormat();
		outNameSet();
	}
}
	
void Qptp::outNameSetFormat()				// format output file name
{
	if(sFile != "") {                           // when user did not press cancel
		sFile = ensurePdfSuffix(sFile);
		if(pdfNameChk(ensurePathSlash(ui.lE_Path->text()) + sFile) == false) {		// check whether pdf name is valid
			sFile = "";
		}
	}
}

void Qptp::outNameSet()						// set name in output file name line edit
{
	disconnect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
	ui.lE_File->setText(sFile);
	sFileOld = sFile;
	connect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
}

void Qptp::outNameLeFileTypeChk()			// check and switch between pdf and jpg when changing creating pdf or jpg
{
	sFile = ui.lE_File->text();
	if(sFile.length() > 3) {
		sD = sFile.right(4);

		if(bPdf == true and sD.contains(".pdf", Qt::CaseInsensitive) == false) {
			sFile.chop(4);
			sFile += ".pdf";
		} else if(bPdf == false and sD.contains(".jpg", Qt::CaseInsensitive) == false ) {
			sFile.chop(4);
			sFile += ".jpg";
		} else {
			return;
		}
		outNameSet();								// only re-set, if something changed
	}
}

void Qptp::outTypeToggled()					// output type toggled: pdf or jpg
{
	outputSetType();
	outNameLeFileTypeChk();
}

// ****************************************************************
// **  create pdf control                                        **
// ****************************************************************
void Qptp::clickedCreate()					// create output file clicked
{
	iErr = 0;
	iRunState = 2;
	updMnTlBr();

//  **  get list of input files to process
	outputGetInputFiles();
		if(slPicsSel.count() == 0) {
			iRunState = 1;
			updMnTlBr();
			return;
		}

//	**  check input files: count OK, 1 or more for pdf or exactly 2 to create jpg, and readable
	if(outputChkInputFiles() == false) {
		runStateReset();
		return;
	}

//	**  create full output file path & name
	if(outputSetOutputFile() == "") {
		runStateReset();
		return;
	}

//  **  check whether output file can be writen in destination dir and whether to overwrite file if file already exists
	if(outputChkOutputFile() == false) {
		runStateReset();
		return;
	}

//  **  set output type
//	outputSetType();

//	**  create output file

//	outNameLeEditFin();						// still needed?
	outputCreatePrep();
}

void Qptp::showErr()						// show any error during pdf creation
{
	if(iErr > 0) {                              // only proceed, if no other widget is shown
		switch(iErr) {
			case 1:                                   // output dir not OK
				messageBox(2, tr("Error - output dir not OK !"), tr("Problem with output directory !\nDoes it exist, is it writable?\nAborting..."));
				break;
//			case 2:                                   // no file selected, must never happen, create button should be disabled in this case !
//				messageBox(2, tr("Error - nothing to do !"), tr("No file is selected - nothing to do.\nThis should never happen.\nAborting..."));
//				break;
			case 8:                                   // error concatenating every 2 pics together, left and right
				messageBox(2, tr("Error: concatenating pictures!"), sErr8);
				break;
			case 32:                                   // error while creating pdf with PDFlib lite
				messageBox(2, tr("Error: pdf creation aborted!"), sErr32);
				break;
    }
  }
}

// ****************************************************************
// **  output preparation                                        **
// ****************************************************************
//  **  get list of input files to process
void Qptp::outputGetInputFiles()
{
	slPicsSel.clear();
	lDirModSelIndexes = ui.tV_FileSys->selectionModel()->selectedIndexes();	// get selection
	if (lDirModSelIndexes.count() == 0) {									// stop if nothing selected - redundant, must never happen
		messageBox(2, tr("Error - nothing to do !"), tr("No file is selected - nothing to do.\nThis should never happen.\nAborting..."));
		return;
	}

//	slPicsToPdf.clear();						// needed ??????????
	for (iD = 0; iD < lDirModSelIndexes.count(); ++iD) {
		slPicsSel.append(model->filePath(lDirModSelIndexes[iD]));
	}
//cout << "outputGetInputFiles() at end : " << QString::number(slPicsSel.count()).toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
}

//  **  check input files: count OK, 1 or more for pdf or exactly 2 to create jpg, and readable - list has at least 1 file
bool Qptp::outputChkInputFiles()
{
	if(ui.rB_Jpg->isChecked() == true and slPicsSel.count() != 2) {		// jpg concatenation, but not exactly 2 files
		messageBox(1, tr("Error"), tr("Jpg concatenation only works with exactly 2 files selected,\nbut currently are %1 file(s) selected!\nAborting...").arg(QString::number(slPicsSel.count())));
		return false;
	}

	QFileInfo fi(slPicsSel.at(0));								// check whether all files are readable
	for(iD = 0; iD < slPicsSel.count(); ++iD) {
		fi.setFile(slPicsSel.at(iD));
		if(fi.isReadable() == false) {			// file not readable - abort
			messageBox(1, tr("Error"), tr("A selected file is not readable:\n%1\nAborting...").arg(slPicsSel.at(iD)));
			return false;
		}
	}

	return true;
}

//  **  create full output file path & name
QString Qptp::outputSetOutputFile()
{
	sOutFullName = "";
	if(ui.cB_InSrcDir->isChecked() == true) {		// set source dir as output dir
		QFileInfo fi(slPicsSel.at(0));
		sOutFullName = fi.absolutePath();
	} else if(ui.lE_Path->text().length() > 0) {	// add output path from lineedit
		sOutFullName += ui.lE_Path->text();
	} else {										// empty output path
		messageBox(1, tr("Error"), tr("Path of output file could not be set!\nAborting..."));
		return "";
	}
	if(sOutFullName.right(1) != "/") {				// ensure, that we have / at end of path
		sOutFullName += "/";
	}
	
	if(ui.lE_File->text().length() > 0) {			// add output file name, use suffic as is, might be user overwritten
		sOutFullName += ui.lE_File->text();
/*		sD = sOutFullName.right(4);
		if(bPdf == true and sD.contains(".pdf", Qt::CaseInsensitive) == false) {
			sOutFullName += sOutFullName + ".pdf";
		} else if(bPdf == false and sD.contains(".jpg", Qt::CaseInsensitive) == false ) {
			sOutFullName += sOutFullName + ".jpg";
		}*/
		return sOutFullName;
	} else {
		messageBox(1, tr("Error"), tr("The output file name could not be set!\nAborting..."));
		return "";									// empty output file name
	}
}

//  **  check whether output file can be writen in destination dir and whether to overwrite file if file already exists
bool Qptp::outputChkOutputFile()
{
	if (ui.cB_InSrcDir->isChecked() == true) {		// check if source/destination dir is writeable
		sD = slPicsSel.at(0);
		sErr = tr("source");
	} else {
		sD = sOutFullName;
		sErr = tr("destination");
	}
	if(destDirChkWri(sD) == false) {
		messageBox(2, tr("Error"), tr("Cannot create output file in %1 directory\n%2\nIt is not writeable.\nAborting...").arg(sErr).arg(fi.absolutePath()));
		return false;
	}

	QFileInfo fi(sOutFullName);						// check if destination file already exists
	if (fi.exists() == true) {
		disconnect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));		
		getCursorPos();
		QMessageBox msgBox;
		msgBox.setWindowTitle(tr("destination file exists !"));
		msgBox.setIcon(QMessageBox::Question);
		msgBox.setText(tr("\nThe destination file\n%1\nalready exists !\nDo you want to overwrite it?").arg(sOutFullName));
		msgBox.addButton(tr("&Yes, Overwrite"), QMessageBox::AcceptRole);
		msgBox.addButton(tr("No, &Cancel"), QMessageBox::RejectRole);
		msgBox.move(iX, iY);
		if (msgBox.exec() == QMessageBox::AcceptRole) {
			connect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
			return true;							// continue with this path/file name - we want to overwrite
		} else {
			connect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
			return false;							// abort
		}
	} else {
		return true;								// does not yet exist, continue
	}
}

void Qptp::outputSetType()					// set output type, pdf or jpg
{
	bPdf = ui.rB_Pdf->isChecked();
}

void Qptp::outputCreatePrep()				// prepare output creation
{
	iRunState = 2;
	updMnTlBr();

//  ** create pdf
	if (bPdf == true) {
		if (ui.cB_PdfConcPic->isChecked() == true and slPicsSel.count() > 1) {
//cout << "pdfCreatePrep() : creating tmpDir " << sTmpDir.toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
			tmpDirCreate();						// create temp dir
			magickAppend();
		} else {								// create pdf directly from selection, no concatenate
			slPicsToPdf = slPicsSel;
			QFileInfo fi(slPicsToPdf.at(0));
			sTmpDir = "";						// empty dir = provided pics have full path n name (= selection)
			magickToPdf();
		}

// ** create jpg
	} else {
		magickAppend();
	}
}

// ******************************************************************
// **  pdf handling excl. magick, create, name chk/create/suggest  **
// ******************************************************************
bool Qptp::pdfNameChk(const QString& sD)	// check whether pdf name is valid
{
	QFileInfo fi(sD);
	if(fi.exists() == true) {
		disconnect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));		
		getCursorPos();
		QMessageBox msgBox;
		msgBox.setWindowTitle(tr("pdf exists !"));
		msgBox.setIcon(QMessageBox::Question);
		msgBox.setText(tr("\nThe pdf\n\n%1\n\nalready exists !\nDo you want to overwrite it?").arg(fi.absoluteFilePath()));
		msgBox.addButton(tr("&Yes, Overwrite"), QMessageBox::AcceptRole);
		msgBox.addButton(tr("No, &Cancel"), QMessageBox::RejectRole);
		msgBox.move(iX, iY);
		if(msgBox.exec() == QMessageBox::AcceptRole) {
			connect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
			return true;
		} else {
			connect(ui.lE_File, SIGNAL(editingFinished()), this, SLOT(outNameLeEditFin()));
			return false;
		}
	} else {
		return true;
	}
}

void Qptp::pdfNameSuggest()					// suggest pdf name acc selection
{
//cout << "pdfNameSuggest() : reached " << endl;      //  *******************  debug  !!!!!!!!!!
	sFile = ui.lE_File->text();
	if(sFile.length() == 0) {
		sFile += tr("result");
	}
	if (ui.rB_Pdf->isChecked() == true) {
		if(sFile.length() < 4 or sFile.right(4).contains(".pdf",Qt::CaseInsensitive) == false) {
			if(sFile.length() > 3 and sFile.right(4).contains(".jpg",Qt::CaseInsensitive) == true) {
				sFile.chop(4);
			}
			sFile += ".pdf";
		}
	} else {
		if(sFile.length() < 4 or sFile.right(4).contains(".jpg",Qt::CaseInsensitive) == false) {
			if(sFile.length() > 3 and sFile.right(4).contains(".pdf",Qt::CaseInsensitive) == true) {
				sFile.chop(4);
			}
			sFile += ".jpg";
		}
	}
	outNameSet();
}

// ********************************************************************************************************************
// **  popup widget handling        **
// ***********************************
void Qptp::wCtrl(int iWToShow)	// show popup widget, if no other is shown: 0=none, 1=about-abouthelp, 2=help-abouthelp, 3=error-abouthelp
{
	if(iWShown == 0) {						// only proceed, if no other widget is shown
		switch(iWToShow) {
			case 1:								// show about widget
				ahwCreate();
				iWShown = 1;
				emit sigAhwShow(iWShown);
				break;
			case 2:								// show help widget
				ahwCreate();
				iWShown = 2;
				emit sigAhwShow(iWShown);
				break;
		}
	}
}

// ********************************************************************************************************************
// **  tmp dir handling                  **
// ****************************************
void Qptp::tmpDirCreate()					// create tmp dir
{
	sTmpDir = tdCreate();
	if(sTmpDir == "") {
		messageBox(3, tr("Error creating tmp dir"),
						tr("Could not create a temporary directory in %1 !\nA tmp dir is mandatory for pdf creation!\nAborting...").arg(QDir::tempPath()));
		quitPrep();
	}
}

void Qptp::tmpDirDelAfterSuccess()			// check whether the output file was created in the temporary directory, if so do not delete
{
	QDir di2(sOutFullName);
	QDir di3(sTmpDir);
	if(di2.absolutePath() == di3.absolutePath()) {
		messageBox(3, tr("Error deleting tmp dir"),
						tr("The output file was created in the temporary directory\n%1\nEnsure, that it will survive a re-start. Dir NOT deleted!").arg(sTmpDir));
		return;
	} else {
		tmpDirDel();
	}
}

bool Qptp::tmpDirDel()						// delete tmp dir
{
	if (sTmpDir != "") {
//cout << "tmpDirDel() : sTmpDir = " << sTmpDir.toStdString() << endl;      //  *******************  debug  !!!!!!!!!!
		if (tdDelTmpDirFiles() == true) {
			if (tdDelTmpDir() == true) {
				sTmpDir = "";
				return true;
			} else {
				messageBox(3, tr("Error - deleting temporary directory!"),
								tr("Could not delete temporary directory %1\nYou may want to see after this by yourself.").arg(sTmpDir));
				return false;
			}
		} else {
			messageBox(3, tr("Error - deleting files in temp dir!"),
							tr("Error while deleting files in the temporary directory %1\nCould not delete the temporary directory either!\nYou may want to see after this by yourself.").arg(sTmpDir));
			return false;
		}
	}
}

// ********************************************************************************************************************
// **  ImageMagick object handling       **
// ****************************************
void Qptp::magickCreate()					// create ImageMagick object (convert command handling)
{
	mgk = new magick();
		connect(this, SIGNAL(sigMagickChk()), mgk, SLOT(magickChk()));
		connect(mgk, SIGNAL(sigChkEnd(int)), this, SLOT(magickChkReturn(int)));
		connect(this, SIGNAL(sigMagickAppend(const QString&, const QStringList&, bool, bool)), mgk, SLOT(magickAppendPrep(const QString&, const QStringList&, bool, bool)));
		connect(mgk, SIGNAL(sigMagickAppendEnded(const QStringList&)), this, SLOT(magickAppendReturn(const QStringList&)));
		connect(this, SIGNAL(sigMagickToPdf(const QString&, const QStringList&, const QString&, const QString&, const QString&)), mgk, SLOT(magickToPdfPrep(const QString&, const QStringList&, const QString&, const QString&, const QString&)));
		connect(mgk, SIGNAL(sigMagickToPdfEnded(const QString&)), this, SLOT(magickToPdfReturn(const QString&)));
	emit sigMagickChk();
}

void Qptp::magickChkReturn(int iD)			// return from checking for ImageMagick's convert command
{
	if(iD == 0) {								// convert found
		initCont();
	} else {
//		iErr = 4;
		messageBox(3, tr("Error - convert not found!"), tr("convert from ImageMagick cannot be found or is not accessable.\nAborting..."));
		quitPrep();
	}
}

void Qptp::magickAppend()					// concatenate 2 images into 1 left + right, pic count > 1 when this is called
{
	statusBarUpd(tr("Concatenate every 2 images into 1 (left to right)..."));
	if(bPdf == true) {
		bConH = true;
	} else {
		if(ui.rB_JpgH->isChecked() == true) {
			bConH = true;
		} else {
			bConH = false;
		}
		sTmpDir = sOutFullName;				// concatenate jpgs - pass the full output path n name as tmp dir for jpg creation
	}
	emit sigMagickAppend(sTmpDir, slPicsSel, bPdf, bConH);	// depending of appending f pdf or jpg, sTmpDir holds either tmp dir or name of resulting jpg
}

void Qptp::magickAppendReturn(const QStringList& slD)	// return from concatenating images, get list of resulting image (jpg) or images (pdf) or empty list=error
{
	slAppendedPics = slD;					// one file if concatenate 2 images into one, or a list of images (or 1) if creating a pdf
	if (bPdf == true) {
		slPicsToPdf = slAppendedPics;						// pdf pics might not always be appended before
		if (slPicsToPdf.count() == 0) {						// error while concatenating pics abort and clean up
			messageBox(3, tr("Error - concatenate images!"), tr("An error occurred while concatenate 2 images to 1!\nStopping..."));
			if (ui.cB_PdfConcPic->isChecked() == true) {
				tmpDirDel();
			}
			runStateReset();							// reset run state and update menu and toolbar
			return;
		}
//  ** continue - create pdf now
		magickToPdf();									
//  ** created jpg - copy from tmp dir to destination and clean up
	} else {
		messageBox(1, tr("Success"), tr("The concatenated picture\n\n%1\n\nseems to be created successfully.").arg(sTmpDir));
		sTmpDir = "";									// was overloaded with destination path/name, reset to avoid trying to delete a dir that does not exist
		runStateReset();								// reset run state and update menu and toolbar
		if (ui.cB_DelOrigPics->isChecked() == true) {
			delPicsOrig();								// delete original pics after successful pdf cration
		}
		dmRefresh();									// refresh file view
	}
}

void Qptp::magickToPdf()					// convert image(s) into 1 pdf
{
	statusBarUpd(tr("Creating 1 pdf from image(s)..."));
	emit sigMagickToPdf(sTmpDir, slPicsToPdf, ui.coB_PdfCompType->currentText(), ui.spB_PdfCompQual->cleanText() ,sOutFullName);
}

void Qptp::magickToPdfReturn(const QString& sD)	// return from converting image(s) into 1 pdf 
{
	sPdfReturn = sD;
	if (sPdfReturn != "" and ui.cB_DelOrigPics->isChecked() == true) {
		delPicsOrig();                              // delete original pics after successful pdf cration
	}

	statusBarUpd(tr("Finished"));

	if (sPdfReturn != "") {
		messageBox(1, tr("Success"), tr("The pdf\n\n%1\n\nseems to be created successfully.").arg(sPdfReturn));
		if (ui.cB_PdfConcPic->isChecked() == true) {	// only delete dir, if real tmp dir (not original dir)
			tmpDirDelAfterSuccess();
		}
	} else {
		messageBox(2, tr("Failure"), tr("It seems, the pdf could not be created."));
		tmpDirDel();
	}
	runStateReset();								// reset run state and update menu and toolbar
	dmRefresh();
}

void Qptp::runStateReset()					// reset run state and update menu and toolbar
{
	iRunState = 1;
	updMnTlBr();
}

QString Qptp::ensurePathSlash(const QString& sD)	// ensure path has a slash at the end
{
	sD2 = sD;
	if(sD2.right(1) != "/") {
		sD2 += "/";
	}
	return sD2;
}

QString Qptp::ensurePdfSuffix(const QString& sD)	// ensure pdf has pdf as suffix
{
//cout << "ensurePdfSuffix() : reached " << endl;      //  *******************  debug  !!!!!!!!!!
	sD2 = sD;
	QFileInfo fi(sD2);
	if (fi.suffix() != "pdf" and fi.suffix() != "PDF") {
		sD2 += ".pdf";
	}
	return sD2;
}

// ****************************************************************
// **  delete pdf, tmp concantenated n original pictures         **
// ****************************************************************
void Qptp::delPdfAfterErr()					// delete unfinished pdf after pictures after pdf creation
{
	f.setFileName(sPdf);
	f.remove();
}

void Qptp::delPicsConc()					// delete temporary concatenated images
{
	if(sTmpDir != "") {
		QDir d(sTmpDir);
		if (slPicsConc.count() > 0) {
			for (iD = 0; iD < slPicsConc.count(); ++iD) {
				d.remove(slPicsConc.at(iD));
			}
		}
	}
}

void Qptp::delPicsOrig()					// delete original pictures after successful pdf/jpg creation
{
	getCursorPos();
	QMessageBox msgBox;
	msgBox.setWindowTitle(tr("OK to delete input pictures ?"));
	msgBox.setIcon(QMessageBox::Question);
	msgBox.setText(tr("\nHate to ask, but are you sure to delete the input pictures?\nThe output file seems to be created successfully.\n"));
	msgBox.addButton(tr("Yes, &Delete"), QMessageBox::AcceptRole);
	msgBox.addButton(tr("No, &Cancel"), QMessageBox::RejectRole);
	msgBox.move(iX, iY);
	if(msgBox.exec() == 0) {
		slDel = slPicsSel;
		QFile fil;
		for (iD = 0; iD < slDel.count(); ++iD) {
			sD = slDel.at(iD);
			fil.setFileName(sD);
			if (fil.remove() == false) {
				messageBox(2, tr("Error removing file !"), tr("Could not remove file:\n %1\nThis might indicate a file permission problem !").arg(sD));
			}
		}
		runStateReset();						// reset run state and update menu and toolbar
		dmRefresh();
	}
}


#include "qpictopdf.moc"
